// file: nios_cache.h
//
// +---------------------------------------------
// | This file contains prototypes and
// | macros for manipulating
// | the Nios memory cache hardware
// |
// | dvb 2002, based on macros by tw 2002
// |
// | These routines are in assembly because they
// | make such intimate use of processor
// | control registers
//
// ex:set tabstop=4:
// ex:set shiftwidth=4:
// ex:set expandtab:

#ifndef _nios_cache_
#define _nios_cache_

#include "excalibur.h"


// +---------------------
// | Definitions:
// |  Bits and Registers

#define np_nios_icache_bit 0x00010000 // bit in control register 0
#define np_nios_dcache_bit 0x00020000 // bit in control register 0

#define np_nios_icache_reg 5          // register to invalidate a line of icache
#define np_nios_dcache_reg 7          // register to invalidate a line of dcache




// +---------------------------
// | void nm_icache_enable(void)
// | void nm_icache_disable(void)
// | void nm_dcache_enable(void)
// | void nm_dcache_disable(void)
// |
// | enables or disables the caches. does not check to see
// | if the cache actually exists. does nothing
// | if cpu does not have that cache.
// |
// | these arent really C routines, but are structured
// | so you invoke them as if they were. for example:
// |
// |    if(time_to_disable_the_data_cache)
// |        {
// |        printf("mother, i am disabling the cache now\n");
// |        nm_dcache_disable();
// |        }
// |

#define nm_icache_enable()                          \
        {                                           \
        int status;                                 \
        asm("rdctl %0" : "=r" (status));            \
        status |= np_nios_icache_bit;               \
        asm("wrctl %0 \n\t nop" : : "r" (status));  \
        }

#define nm_icache_disable()                         \
        {                                           \
        int status;                                 \
        asm("rdctl %0" : "=r" (status));            \
        status &= ~np_nios_icache_bit;              \
        asm("wrctl %0 \n\t nop" : : "r" (status));  \
        }

#define nm_dcache_enable()                          \
        {                                           \
        int status;                                 \
        asm("rdctl %0" : "=r" (status));            \
        status |= np_nios_dcache_bit;               \
        asm("wrctl %0 \n\t nop" : : "r" (status));  \
        }

#define nm_dcache_disable()                         \
        {                                           \
        int status;                                 \
        asm("rdctl %0" : "=r" (status));            \
        status &= ~np_nios_dcache_bit;              \
        asm("wrctl %0 \n\t nop" : : "r" (status));  \
        }


// +-------------------------------
// | void nm_icache_invalidate_line(void *address)
// | void nm_dcache_invalidate_line(void *address)
// |
// | clears the line in the cache which *could* correspond
// | to the address given. the line is cleared, even
// | if it doesnt happen to be presently holding that
// | address cached up.
// |
// | the address you pass is the byte address of memory.
// |
// | (deep down, that cache entry is marked as "empty"
// | if it had previously been marked as "holding something",
// | based on the low bits of the address passed.)
// |

#define nm_icache_invalidate_line(byte_address) \
        asm("pfx 5 \n\t wrctl %0" : : "r" (byte_address));

#define nm_dcache_invalidate_line(byte_address) \
        asm("pfx 7 \n\t wrctl %0" : : "r" (byte_address));


// +----------------------------------
// | Routine Prototypes
// |
// | Use these routines (genuine C routines)
// | to operate on more than one cache line
// | at a time
// |

void nr_icache_init(void);
void nr_dcache_init(void);
void nr_icache_invalidate_lines(void *low_address, void *high_address);
void nr_dcache_invalidate_lines(void *low_address, void *high_address);
    
#endif // _nios_cache_


// +------------------------------------------
// | early notes from the original implementation
// |
// | possibly good reading, possibly obsolete!
// |

// Note for both Cache's on address alignment for *_invalidate_line macros:
// It's trivial in the hardware to automagically select the appropriately
// aligned bits of the wrctl registers so that 'aligned_address's do not need
// to be computed here manually.  For example:
// NY ICache lines are 16 bits wide, so you could write any byte address to
// ctl reg 5, and simple send reg5[max:1] to the ic_invalid_set[max-1:0] port.
//
// It just doesn't happen to be in the code yet, among a few other things.

/* Instruction Cache */

// Note: nasys_icache_lines is currently static, but will ultimately be
// generated by the GUI.  Likewise for nasys_icache_alignment, which is 
// 2 bytes for NY, 4 bytes for NJ

// end of file
